#include <_Custom_Operacional/Estrategia/Strategy.mqh>
#include <_Custom_Charts/Indicadores/MovingAverage.mqh>

class MovingAverageStg : public Strategy{
private:
    //Ponteiro do indicador da strategy
    MovingAverage* mAvgIndicador;
    //Relação com os buffers
    ENUM_MARKET_TO_BUFFER_RELATION relatAverage;
    ENUM_MARKET_TO_BUFFER_RELATION relatSupEdge;
    ENUM_MARKET_TO_BUFFER_RELATION relatInfEdge;

    int totalIndicacoesCompra;
    int totalIndicacoesVenda;

public:
    MovingAverageStg(){};
    ~MovingAverageStg(){};

    //Passando valores do NsVPR
    void set(int periodo_, double desvio_, ENUM_MA_TYPE tipo_, int espelho_);
    //Método genérico a ser implementado pelas classes filhas da definição do indicador da strategy
    virtual void set();
    //Define a relação do mercado com os buffers do indicador
    virtual void setBuffersRelation();
    //Método genérico a ser implementado pelas classes filhas para retorno da indicação atual da strategy
    virtual ENUM_STRATEGY_SIGN getSign();
    //Método genérico para setar o valor de relação dos buffers para não relacionados
    virtual void setUnrelatedBuffers();
    //Reinicia a strategy
    void restartStrategy();

    int getTotalIndicacoesCompra() { return this.totalIndicacoesCompra; };
    int getTotalIndicacoesVenda() { return this.totalIndicacoesVenda; };
};

void MovingAverageStg::set(int periodo_, double desvio_, ENUM_MA_TYPE tipo_, int espelho_){
    ZeroMemory(this.mAvgIndicador);
    this.mAvgIndicador = new MovingAverage;
    mAvgIndicador.set(periodo_, desvio_, tipo_, espelho_);
    this.setUltimoSinal(NONE_SIGN);
    this.restartStrategy();
}

void MovingAverageStg::set(){
    this.set(60, 20, CLOSE, 1);
}

void MovingAverageStg::setBuffersRelation(){
    relatAverage = getInitialBufferRelation(mAvgIndicador.getBufferLast(MA_BUFFER));
    relatSupEdge = getInitialBufferRelation(mAvgIndicador.getBufferLast(MA_UP_DEVIATION));
    relatInfEdge = getInitialBufferRelation(mAvgIndicador.getBufferLast(MA_DOWN_DEVIATION));
}

ENUM_STRATEGY_SIGN MovingAverageStg::getSign(){
    //Contadores de indicação
    int indicacoesCompra = 0;
    int indicacoesVenda = 0;
    relatAverage = getBufferRelation(mAvgIndicador.getBufferLast(MA_BUFFER), relatAverage);
    relatSupEdge = getBufferRelation(mAvgIndicador.getBufferLast(MA_UP_DEVIATION), relatSupEdge);
    relatInfEdge = getBufferRelation(mAvgIndicador.getBufferLast(MA_DOWN_DEVIATION), relatInfEdge);
    //Desvios
    if(relatSupEdge == CROSSED_UP) //Cruzamento do desvio superior para cima é uma compra média
        indicacoesCompra += 2;
    if(relatSupEdge == CROSSED_DOWN) //Cruzamento do desvio superior para baixo é uma venda média
        indicacoesVenda += 2;
    if(relatInfEdge == CROSSED_UP) //Cruzamento do desvio inferior para cima é uma compra média
        indicacoesCompra += 2;
    if(relatInfEdge == CROSSED_DOWN) //Cruzamento do desvio inferior para baixo é uma venda média
        indicacoesVenda += 2;
    //Media
    if(relatAverage == CROSSED_UP) //Cruzamento do média para cima é uma compra fraca
        indicacoesCompra += 1;
    if(relatAverage == CROSSED_DOWN) //Cruzamento do média para baixo é uma venda fraca
        indicacoesVenda += 1;

    totalIndicacoesCompra += indicacoesCompra;
    totalIndicacoesVenda += indicacoesVenda;
    //Verificação das indicações e retorno
    if(indicacoesCompra > 0 || indicacoesVenda > 0){
        if(indicacoesCompra > indicacoesVenda){
            this.setUltimoSinal(this.getBuySignLevel(indicacoesCompra));
            return this.getUltimoSinal();
        }else{
            this.setUltimoSinal(this.getSellSignLevel(indicacoesVenda));
            return this.getUltimoSinal();
        }
    }else
        return NONE_SIGN;
}

void MovingAverageStg::setUnrelatedBuffers(){
    relatAverage = UNRELATED_TO_BUFFER;
    relatSupEdge = UNRELATED_TO_BUFFER;
    relatInfEdge = UNRELATED_TO_BUFFER;
}

void MovingAverageStg::restartStrategy(){
    this.setUnrelatedBuffers();
}